// Controller for the Syncronization (a.k.a. Asyncronous) FIFO
/* This module must ensure that the fifo is not empty during normal 
   operation (as jitter could shift everything by 1 at any time).
   Thus the fifo content should alternate as 1-2-1-2-1-2.
   This should happen every time that:
   - the link is established  
   - the xilinx is reconfigured 
   - the DLL re-locks
   - there is a latency shift (full, empty, FifoStatus)
   There is no explicit reset (remember that the xilinx
   automatically reset at the end of the configuration phase).
   There are no constraint on the execution time of this procedure,
   but the data-path latency must be minimized (thus fifo content should
   be 1-2-1-2 or 2-3-2-3 rather then 3-4-3-4).
   Note a 1-tick uncertainity due to the random phase relations btw clocks.
   When the link is not establisched or there are spare errors, the Er
   and DV flags must be available for readout.     */

`timescale 1ns / 10ps
module FifoCtr (clk2x, RX_Clk, RX_ER, LatTrim, full, empty,
                FifoStatus,  FifoRestart, LW, WrEn, RdEn);

input  clk2x, RX_Clk, RX_ER, LatTrim, full, empty, FifoRestart;
input [3:0] FifoStatus;
output LW, WrEn, RdEn;         reg    LW, WrEn, RdEn;
reg WrEn_clk2x, WrEn_d;
reg [2:0] lc; // link-count: 7 consequtive valid data or idle words
reg [1:0] ec; // error count
reg [1:0] LinkState;
parameter
   LINK_LOST        = 0,  
   LINK_ESTABLISHED = 1,
   LINK_CHECK       = 2;
reg [3:0] FifoState;    
parameter
   IDLE          =   0,
   CLEAR         =   1,             
   START_WR      =   2,
   NOOP1         =   3, 
   NOOP2         =   4,
   NOOP3         =   5, 
   NORMAL        =   6;

// Monitor the link and decide its status, inspired by Fig.6 of TLK2501 DataSheet
always @(posedge RX_Clk)  
  case(LinkState)

   LINK_LOST: begin   // search 7 consecutive words with RX_ER=0
     ec <= 0;
     if ( (RX_ER==0) & (lc!=3'd7) )   lc <= lc+1;
     if (RX_ER==1)                    lc <= 0;
     if (lc==3'd7)     LinkState <= LINK_ESTABLISHED;
   end  // LINK_LOST

   LINK_ESTABLISHED:   begin
     if (RX_ER==1)  begin
       LinkState <= LINK_CHECK;
       lc <=0;
       ec <=0;
     end  
   end // LINK_ESTABLISHED

   LINK_CHECK:  begin
    if ( (RX_ER==0) & (lc!=3'd7) )    lc <= lc+1;
    if (RX_ER==1) begin
       lc <= 0;
       ec <= ec+1;
       end
    if (ec==2'd3)        LinkState <= LINK_LOST;
    else if (lc==3'd7)   LinkState <= LINK_ESTABLISHED;
   end  // LINK_CHECK
   
  endcase

// WrEn crosses the clock domains 
always @(posedge RX_Clk) begin
   WrEn_d <= WrEn_clk2x;
   WrEn   <= WrEn_d;
end  // always

// Preset or "flush"  procedure
always @(posedge clk2x) 
  case(FifoState)
   IDLE: begin // while link is not established, let data&flags go thru
      WrEn_clk2x <= 1;     RdEn <= 1;
      LW <= 0; 
      if (LinkState == LINK_ESTABLISHED)   begin
            WrEn_clk2x <= 0;
            FifoState <= CLEAR;
      end //if
   end //IDLE
   CLEAR:   
      if (empty)  begin
         RdEn <= 0;
         FifoState <= START_WR;
      end
   START_WR: 
      if (empty) // a redundant check (empty depends on 2 async. clocks)
          begin   
             FifoState <= NOOP1;
             WrEn_clk2x <= 1;
          end
      else   FifoState <= IDLE;
   NOOP1:  // this state offset the delay to synchronize WrEn
      FifoState <= NOOP2;
   NOOP2:  // this state ~offset the delay to synchronize WrEn
      FifoState <= NOOP3;
   NOOP3:  // this state for the fifo to fill
      FifoState <= NORMAL;
   NORMAL: begin
      RdEn <= 1;
      if (full | empty | FifoStatus > 'd4)  LW <= 1; // Latency Warning 
      if ((LinkState == LINK_LOST) | (FifoRestart) )  FifoState <= IDLE;
      if (LatTrim & LW)  FifoState <= IDLE;
   end // NORMAL
  endcase

endmodule