//*********************************************************************************
// FILENAME:  top.v         AUTHORS  :   Drew, Tullio
// DESCRIPTION :   Top level of the HTR Main FPGA.
// CLOCK inputs: 
// - TTC_Clk (from jittery TTCrx on the same HTR board of the Xilinx)
// - RxClk   (low-jitter 40 MHz from a QPLL common to a VME crate)
/********   To Be Done:  ****
- review reset logic
********/
//**********************  REVISION  HISTORY  *********************************
/*  HTRev4_v25 built from HTRev4_v19
- Zero-Sup on compressed Et, with individual Thresholds per channel
- reviewed timing of OD error flag (now it is asserted on the guilty
  event and on the next one).
- In LocBusSlave:   if (NS == 0) NS <= 'd20; 
- BCR set BCN to a programmable value "SetBCN"
CHANGES from HTRev4_v17
- fixed confusion PatternMode vs CounterMode
CHANGES from HTRev4_v17
- DaqFormatter input CounterModeSel ORed with PatternSelD (as in data format)
- modified Ext_Header 7 and 8 as in Mar05 data format
CHANGES from HTRev4_v16
- .CLKFB(clk2x) changed to .CLKFB(clk2x_dll). (compliant with ISE6.3)
- moved hamcode_gen inside DaqPath; eliminated DaqOrHistoMux
  CHANGES from HTRev4_v15
- For TTC_CK, IBUFG replaced with BUFGP: skew improved from 9ns to 0.4ns!
- Similar change on TTC_BrcstStr, using attribute.
- implemented OW initial DaqCntr, instead of OvF.
  CHANGES from HTRev4_v14
- Feb05 DCC data format (implemented LE bit too)
- error bits: OD, LE, LW use channel mask
- reset_TP registred, now takes effect after ComprLUT
CHANGES from HTRev4_v13
- fixed MuonBit in bypass mode (in TpgSingleChannel.v)
 */

`include "define.v"

module top
 ( clk40_P,   clk40_N,      BC0_p,  BC0_n,   SD,
  Ch1_RX_D,   Ch1_RX_ER,    Ch1_RX_DV,    Ch1_RX_Clk_pad,   
  Ch2_RX_D,   Ch2_RX_ER,    Ch2_RX_DV,    Ch2_RX_Clk_pad,  
  Ch3_RX_D,   Ch3_RX_ER,    Ch3_RX_DV,    Ch3_RX_Clk_pad,  
  Ch4_RX_D,   Ch4_RX_ER,    Ch4_RX_DV,    Ch4_RX_Clk_pad,  
  Ch5_RX_D,   Ch5_RX_ER,    Ch5_RX_DV,    Ch5_RX_Clk_pad,  
  Ch6_RX_D,   Ch6_RX_ER,    Ch6_RX_DV,    Ch6_RX_Clk_pad,      
  Ch7_RX_D,   Ch7_RX_ER,    Ch7_RX_DV,    Ch7_RX_Clk_pad,  
  Ch8_RX_D,   Ch8_RX_ER,    Ch8_RX_DV,    Ch8_RX_Clk_pad,  
  TP1_Data,  TP2_Data,  TP3_Data,  TP4_Data,  TP5_Data,  TP6_Data,
  TP7_Data,  TP8_Data,  TP9_Data,  TP10_Data, TP11_Data, TP12_Data,
  TP13_Data, TP14_Data, TP15_Data, TP16_Data, TP17_Data, TP18_Data,
  TP19_Data, TP20_Data, TP21_Data, TP22_Data, TP23_Data, TP24_Data,  
  TTC_DbErrStr,  TTC_SinErrStr,
  TTC_Reset_del,
  TTC_BCntRes,   TTC_Brcst,     TTC_BrcstStr,  TTC_EvCntRes,
  TTC_L1Accept,  TTC_CK40Des1,  TTC_Ready,     TTC_Reset_b,
  LocAddr,       LocData,       LocCS_b,       LocWRITE_b,
  L2frame,       L2clk,         LED,  
  TI_EN,         RotarySwitch,  TestPoint,    ANDsig1      );

 input BC0_p, BC0_n; //differential BC0
 input [8:1]   SD;
 input [15:0]  Ch1_RX_D,  Ch2_RX_D,  Ch3_RX_D,  Ch4_RX_D;  
 input [15:0]  Ch5_RX_D,  Ch6_RX_D,  Ch7_RX_D,  Ch8_RX_D; 
 input         Ch1_RX_ER, Ch1_RX_DV, Ch1_RX_Clk_pad;
 input         Ch2_RX_ER, Ch2_RX_DV, Ch2_RX_Clk_pad;
 input         Ch3_RX_ER, Ch3_RX_DV, Ch3_RX_Clk_pad;
 input         Ch4_RX_ER, Ch4_RX_DV, Ch4_RX_Clk_pad;
 input         Ch5_RX_ER, Ch5_RX_DV, Ch5_RX_Clk_pad;
 input         Ch6_RX_ER, Ch6_RX_DV, Ch6_RX_Clk_pad;
 input         Ch7_RX_ER, Ch7_RX_DV, Ch7_RX_Clk_pad;
 input         Ch8_RX_ER, Ch8_RX_DV, Ch8_RX_Clk_pad;

 input [7:2] TTC_Brcst;
 input TTC_BrcstStr; // BrcstStr1, related to CK40Des1 and Brcst[5:2]
 // synthesis attribute clock_buffer of TTC_BrcstStr is bufgp;
 input TTC_BCntRes, TTC_EvCntRes, TTC_L1Accept;
 input TTC_CK40Des1, TTC_Ready, TTC_DbErrStr,  TTC_SinErrStr;
 input TTC_Reset_del;
  
 //   Local Bus  port:
 input LocCS_b;  // Chip Select
 input LocWRITE_b;  // LOW = Write ; HIGH = Read
 input [`LB_Addr_Size-1:0] LocAddr;  // The Address bus.
 inout [`REG_WIDTH-1:0]    LocData;  // The bidir Data bus 

 output [23:0]  L2frame;     // DAQ-path OUTPUT
 output [`TRIGGER_PATH_DATA_WIDTH - 1 : 0 ]   // Trigger-path OUTPUT
      TP1_Data,  TP2_Data,  TP3_Data,  TP4_Data,  TP5_Data,  TP6_Data,
      TP7_Data,  TP8_Data,  TP9_Data,  TP10_Data, TP11_Data, TP12_Data,
      TP13_Data, TP14_Data, TP15_Data, TP16_Data, TP17_Data, TP18_Data,
      TP19_Data, TP20_Data, TP21_Data, TP22_Data, TP23_Data, TP24_Data;

 output TTC_Reset_b;   // reset to the TTCrx chip
 output [8:1] TI_EN;   // individual enable for TLK2501
 output ANDsig1;   

//  *******************************************************
//    **************  TESTING OUTPUTS  *****************

 output [4:1]  LED;     // 4 LEDs on front-panel
 output [6:1]  TestPoint;   // NB: TestPoint[4] not connected on PCB
 input  [4:1]  RotarySwitch; // on front-panel   

//  *******************************************************
//         ***********  INTERNAL SIGNALS ********

 wire Hard_rst, Soft_rst, Start, Stop, L1A, TTC_BC0, AsyncFifoRst,
      DLL_lock, run_status, EventBufferEmpty,
      bypass, CounterModeSel, PatternMode, PatternSel, LatTrim,  
      LB_Tri, L2IDLE_b, TTC_TestEnable, ClockSelect,
      resetTP1, resetTP2, resetTP3, resetTP4,
      resetTP5, resetTP6, resetTP7, resetTP8,
      Fib1BZero, Fib2BZero, Fib3BZero, Fib4BZero, 
      Fib5BZero, Fib6BZero, Fib7BZero, Fib8BZero;  //it's one BZero per fiber
 wire [1:0]  StatusBits ;
 wire [2:0] LB_STATE; // state of FSM in LocBusSlave, for testing
 wire [3:0] AFifo1Status, AFifo2Status, AFifo3Status, AFifo4Status, 
            AFifo5Status, AFifo6Status, AFifo7Status, AFifo8Status;   
 wire [4:0] NS, NPS; //Programmable # of Samples and Pre-Samples (per L1A)
 wire [7:0] LastTTCcommand;    // in case of testing 
 wire [8:1] AFifoEmpty, AFifoFull, LW; // Conditions of the 8 Input FIFOs.
 wire [8:1] WrEn, RdEn;
 wire [10:0]
   QIE1Data,   QIE2Data,   QIE3Data,   QIE4Data,   QIE5Data,   QIE6Data,
   QIE7Data,   QIE8Data,   QIE9Data,   QIE10Data,  QIE11Data,  QIE12Data,
   QIE13Data,  QIE14Data,  QIE15Data,  QIE16Data,  QIE17Data,  QIE18Data,
   QIE19Data,  QIE20Data,  QIE21Data,  QIE22Data,  QIE23Data,  QIE24Data;    
 wire  [17:0] Pattern1, Pattern2, Pattern3, Pattern4,
              Pattern5, Pattern6, Pattern7, Pattern8,                  
   Threshold1,  Threshold2,  Threshold3,  Threshold4,  Threshold5,  Threshold6,
   Threshold7,  Threshold8,  Threshold9,  Threshold10, Threshold11, Threshold12, 
   Threshold13, Threshold14, Threshold15, Threshold16, Threshold17, Threshold18,
   Threshold19, Threshold20, Threshold21, Threshold22, Threshold23, Threshold24; 
 wire [15:0]  ProgInputDelay,
      ErrAcc1, ErrAcc2, ErrAcc3, ErrAcc4, ErrAcc5, ErrAcc6, ErrAcc7 , ErrAcc8;
 wire [23:0]  L2frameINT;  // DAQ-path output before series resistors.
 wire [24:1] QieChMask;  // mask QIE channels in TPG (needed for SLB histogramming)
 wire [`TRIGGER_PATH_DATA_WIDTH - 1 : 0]  // TriggerPath output before series resistors.
  TP1_DataINT,  TP2_DataINT,  TP3_DataINT,  TP4_DataINT,  TP5_DataINT,  TP6_DataINT,
  TP7_DataINT,  TP8_DataINT,  TP9_DataINT,  TP10_DataINT, TP11_DataINT, TP12_DataINT,
  TP13_DataINT, TP14_DataINT, TP15_DataINT, TP16_DataINT, TP17_DataINT, TP18_DataINT,
  TP19_DataINT, TP20_DataINT, TP21_DataINT, TP22_DataINT, TP23_DataINT, TP24_DataINT;
 wire [`MAIN_DATA_WIDTH- 1 : 0]   DataOut;
 wire [`REG_WIDTH-1:0] HTRsubmodN, PipeLength, LB_DOUT;
 wire [`SpyFIFO_WIDTH-1:0]  //  FE buses before the 2->3 mux
      InputSpyData1,  InputSpyData2,    InputSpyData3,  InputSpyData4,
      InputSpyData5,  InputSpyData6,   InputSpyData7,  InputSpyData8;
 wire [`EVN_WIDTH-1:0] EvN, CTrEvN;
 wire [`OrN_WIDTH-1:0] OrN;  
 wire [`BCN_WIDTH-1:0] BCN,
       BCNofFib1BZero, BCNofFib2BZero, BCNofFib3BZero, BCNofFib4BZero, 
       BCNofFib5BZero, BCNofFib6BZero, BCNofFib7BZero, BCNofFib8BZero;
 reg  [26:0] clk_scaled;  // heart-beat
 reg  ErStratch, Er_d, Er_2d, Er_3d, DV; // for LEDs
      // LUTs stuff:                                                         
 wire  [10:0]   InLUT_Data, 
   InLut1DOut,  InLut2DOut,  InLut3DOut,  InLut4DOut,  InLut5DOut,  InLut6DOut,  
   InLut7DOut,  InLut8DOut,  InLut9DOut,  InLut10DOut, InLut11DOut, InLut12DOut,
   InLut13DOut, InLut14DOut, InLut15DOut, InLut16DOut, InLut17DOut, InLut18DOut, 
   InLut19DOut, InLut20DOut, InLut21DOut, InLut22DOut, InLut23DOut, InLut24DOut; 
 wire  [6:0]  InLUT_Addr;
 wire  [24:1] InLUT_WE;  
 wire  InLUT_Mode;    
 wire  [7:0]      OutLUT_Data,
   OutLUT_DOut1,  OutLUT_DOut2,  OutLUT_DOut3,  OutLUT_DOut4,
   OutLUT_DOut5,  OutLUT_DOut6,  OutLUT_DOut7,  OutLUT_DOut8,
   OutLUT_DOut9,  OutLUT_DOut10, OutLUT_DOut11, OutLUT_DOut12,
   OutLUT_DOut13, OutLUT_DOut14, OutLUT_DOut15, OutLUT_DOut16,
   OutLUT_DOut17, OutLUT_DOut18, OutLUT_DOut19, OutLUT_DOut20,
   OutLUT_DOut21, OutLUT_DOut22, OutLUT_DOut23, OutLUT_DOut24;      
 wire  [9:0]  OutLUT_Addr;
 wire  [24:1] OutLUT_WE;  
 wire  OutLUT_Mode; 

//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

// XST was stupid at handling clocks and clock buffer, so we instantiate
// all the buffers and inihbit XST from changing them with the attribute.
 input  clk40_P, clk40_N;  // differential CLK input
 wire   RxClk, SelectedClk, clk, clk2x, clk0, clk2x_dll, clk90, clk180;
 wire   [7:0] DLL_status;   // DLL_status[1] asserted if .CLKIN stopped
 reg    ResetDLL, DLL_lock_D;
 reg    [14:0] RstCnt; // counter for proper reset of DLL 
 reg    [2:0] UnlockCnt = 0; // counter for proper reset of DLL 
 output L2clk; 

 //  Instantiation of Xilinx primitives
  IBUFGDS_LVPECL_33  DifClkbuf (.I(clk40_P), .IB(clk40_N), .O(RxClk) );
  BUFGP              TTCclkbuf (.I(TTC_CK40Des1), .O(TTC_CK) );
  // BUFGP = IBUFG + BUFGMUX
  BUFGMUX            SystemClockMux   ( .S(ClockSelect),
                     .I0(TTC_CK),  .I1 (RxClk),  .O (SelectedClk)  );
//ConfigRst asserted for ~3 clock cycles (tested), see also:
//www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
// Note that FDS is set at power-up.
  FDS flop1 (.D(1'b0), .C(SelectedClk), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SelectedClk), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SelectedClk), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SelectedClk), .Q(out4)); 
  assign ConfigRst = (out2 | out3 | out4); 

  DCM # (
    .CLK_FEEDBACK("2X") 
	 ) 
  DCM1 (
    .CLKIN(SelectedClk), .CLKFB(clk2x_dll),  .RST(ConfigRst),  
    .CLK0(clk0),   .CLK90(clk90),  .CLK180(clk180),  .CLK270(),
    .CLK2X(clk2x_dll),  .CLK2X180(),
    .CLKDV(),   .CLKFX(),   .CLKFX180(),
    .LOCKED(DLL_lock),  .STATUS(DLL_status),
    .DSSEN(1'b0),
    .PSCLK(1'b0),   .PSEN(1'b0),   .PSINCDEC(1'b0), .PSDONE() 
	 );

/* If tests show that DCM looses lock, consider to
   increase the FACTORY_JF setting to 'FFFF  (Xilinx Answer 13756).
   Anyway lab test in Dec03 showed that the DLL relock spontaneously
   when the TTCrx was loosing lock and then relocking   */
/* If there is the chance that TTC_CK stops completly, the LOCKED signal
   will not update, consider the status bit "CLK_IN_STOPPED" of DCM
   I do not implement it as I don't expect this occurence, although
   trivial mechanical troubles of the TTC-PMC mezzanine could do it. 
   Note that if the TTCrx looses lock, TTC_clk will keep flipping at 
   the TTCrx natural oscillation frequency    */

// ResetDLL has some "feedback" problems when connected to DCM
// Eliminate it for now.
/*
 always @ (posedge TTC_CK)  begin //TTC_CK should toggle even if no cable 
    if (DLL_lock == 0) begin
         RstCnt <= RstCnt + 1;
         if (RstCnt == 15'h 7FFF)
               ResetDLL <= 1;  // reset no more then every 2^15 25ns
         else     ResetDLL <= 0;
    end
    DLL_lock_D <= DLL_lock;
    if (Hard_rst)  UnlockCnt <= 0;
    else if (DLL_lock_D & (~DLL_lock) & (UnlockCnt != 3'b111)) //hi->lo transition
             UnlockCnt <= UnlockCnt + 1;   // increment (stop at max count)
 end //always 
*/
 BUFG CK40BUF (.I(clk0),  .O(clk));      // Use this output as internal clock 
 //synthesis attribute clock_buffer of clk0 is "none" 
 BUFG CK2XBUF (.I(clk2x_dll), .O(clk2x)); // clock for data inputs
 //synthesis attribute clock_buffer of clk2x_dll is "none" 
 OBUF_LVDCI_DV2_25  L2CKBUF  (.I(clk90),  .O(L2clk) ); // clk to off-chip National Tx 
 //synthesis attribute clock_buffer of clk90 is "none" 

// the following 8 instantiations are to make sure that the 8 recovered clocks
// are treated homogenously. Add constraint USELOWSKEWLINES on .ucf
 IBUFG RXCK1BUF (.I(Ch1_RX_Clk_pad), .O(Ch1_RX_Clk));
 //synthesis attribute clock_buffer of Ch1_RX_Clk_pad is "none" 
 IBUFG RXCK2BUF (.I(Ch2_RX_Clk_pad), .O(Ch2_RX_Clk));
 //synthesis attribute clock_buffer of Ch2_RX_Clk_pad is "none" 
 IBUFG RXCK3BUF (.I(Ch3_RX_Clk_pad), .O(Ch3_RX_Clk));
 //synthesis attribute clock_buffer of Ch3_RX_Clk_pad is "none" 
 IBUFG RXCK4BUF (.I(Ch4_RX_Clk_pad), .O(Ch4_RX_Clk));
 //synthesis attribute clock_buffer of Ch4_RX_Clk_pad is "none" 
 IBUFG RXCK5BUF (.I(Ch5_RX_Clk_pad), .O(Ch5_RX_Clk));
 //synthesis attribute clock_buffer of Ch5_RX_Clk_pad is "none" 
 IBUFG RXCK6BUF (.I(Ch6_RX_Clk_pad), .O(Ch6_RX_Clk));
 //synthesis attribute clock_buffer of Ch6_RX_Clk_pad is "none" 
 IBUFG RXCK7BUF (.I(Ch7_RX_Clk_pad), .O(Ch7_RX_Clk));
 //synthesis attribute clock_buffer of Ch7_RX_Clk_pad is "none" 
 IBUFG RXCK8BUF (.I(Ch8_RX_Clk_pad), .O(Ch8_RX_Clk));
 //synthesis attribute clock_buffer of Ch8_RX_Clk_pad is "none" 

//       *************  END OF CLOCK SECTION  *****************
//  ******************************************************************

//          *********  TEST POINTS and LEDs  ****************
// ASSIGNement to Test Outputs 
 // *** LED ON <=> signal '0'  => invert signals ****
 assign LED[4:1] = {~ErStratch, ~DV, ~run_status, clk_scaled[26] & TTC_Ready & DLL_lock};
 wire [7:0] MuxInOne   = {ResetDLL, DLL_lock, LocWRITE_b, AsyncFifoRst, 
                          Start, TTC_BC0, TTC_BrcstStr, L1A};
 wire [7:0] MuxInTwo   = RdEn[8:1];
 wire [7:0] MuxInThree = WrEn[8:1];
 wire [7:0] MuxInFive  = {clk, BCN[1:0], TTC_BCntRes, LastTTCcommand[1:0],
                           TTC_EvCntRes, TTC_TestEnable};
 assign TestPoint[1]   = MuxInOne[~RotarySwitch];          
 assign TestPoint[2]   = MuxInTwo[~RotarySwitch];          
 assign TestPoint[3]   = MuxInThree[~RotarySwitch];       
 assign TestPoint[4]   = 1'b0; // not connected on the board           
 assign TestPoint[5]   = MuxInFive[~RotarySwitch];          
 assign TestPoint[6]   = clk2x;          
 
 always @ (posedge clk2x)    
  begin
    clk_scaled <= clk_scaled + 1;
    case (~RotarySwitch)    // rotary switch to LEDs 
      4'd0: begin
            Er_d      <= InputSpyData1[17]; Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData1[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData1[16];
      end
      4'd1: begin
            Er_d      <= InputSpyData2[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData2[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData2[16];
      end
      4'd2: begin
            Er_d      <= InputSpyData3[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData3[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData3[16];
      end
      4'd3: begin
            Er_d      <= InputSpyData4[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData4[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData4[16];
      end
      4'd4: begin
            Er_d      <= InputSpyData5[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData5[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData5[16];
      end
      4'd5: begin
            Er_d      <= InputSpyData6[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData6[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData6[16];
      end
      4'd6: begin
            Er_d      <= InputSpyData7[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData7[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData7[16];
      end
      4'd7: begin
            Er_d      <= InputSpyData8[17];  Er_2d <= Er_d; Er_3d <= Er_2d;
            ErStratch <= InputSpyData8[17] | Er_d | Er_2d | Er_3d;
            DV        <= InputSpyData8[16];
      end
      4'd8: begin //global  excluding link 1 and 8 (as in TB)
            ErStratch <= InputSpyData2[17] | InputSpyData3[17] | InputSpyData4[17] | 
                         InputSpyData5[17] | InputSpyData6[17] | InputSpyData7[17] ;
            DV        <= InputSpyData2[16] & InputSpyData3[16] & InputSpyData4[16] & 
                         InputSpyData5[16] & InputSpyData6[16] & InputSpyData7[16] ;
      end
      4'd9: begin //global
            ErStratch <= InputSpyData1[17] | InputSpyData2[17] | InputSpyData3[17] |
                         InputSpyData4[17] | InputSpyData5[17] | InputSpyData6[17] |
                         InputSpyData7[17] | InputSpyData8[17];
            DV        <= InputSpyData1[16] & InputSpyData2[16] & InputSpyData3[16] |
                         InputSpyData4[16] & InputSpyData5[16] & InputSpyData6[16] |
                         InputSpyData7[16] & InputSpyData8[16];
      end
    endcase
 end


//  **********************************************************
//    *************    MODULE  INSTANTIATION  *************

  IBUFDS_LVPECL_33 difBC0buf (.I(BC0_p), .IB(BC0_n), .O(BC0_INT) );

// input_stage port map. Format:  .InternalPortName(ext_port_expr)
 InputStage     InputStage
 ( .clk(clk),    .clk2x(clk2x),    .Reset(Soft_rst),
   .AsyncFifoRst(AsyncFifoRst),    .DLL_lock(DLL_lock), 
   .Ch1_RX_D(Ch1_RX_D),      .Ch1_RX_Clk(Ch1_RX_Clk), 
   .Ch1_RX_ER(Ch1_RX_ER),    .Ch1_RX_DV(Ch1_RX_DV), 
   .Ch2_RX_D(Ch2_RX_D),      .Ch2_RX_Clk(Ch2_RX_Clk), 
   .Ch2_RX_ER(Ch2_RX_ER),    .Ch2_RX_DV(Ch2_RX_DV), 
   .Ch3_RX_D(Ch3_RX_D),      .Ch3_RX_Clk(Ch3_RX_Clk), 
   .Ch3_RX_ER(Ch3_RX_ER),    .Ch3_RX_DV(Ch3_RX_DV), 
   .Ch4_RX_D(Ch4_RX_D),      .Ch4_RX_Clk(Ch4_RX_Clk), 
   .Ch4_RX_ER(Ch4_RX_ER),    .Ch4_RX_DV(Ch4_RX_DV), 
   .Ch5_RX_D(Ch5_RX_D),      .Ch5_RX_Clk(Ch5_RX_Clk),
   .Ch5_RX_ER(Ch5_RX_ER),    .Ch5_RX_DV(Ch5_RX_DV), 
   .Ch6_RX_D(Ch6_RX_D),      .Ch6_RX_Clk(Ch6_RX_Clk),  
   .Ch6_RX_ER(Ch6_RX_ER),    .Ch6_RX_DV(Ch6_RX_DV), 
   .Ch7_RX_D(Ch7_RX_D),      .Ch7_RX_Clk(Ch7_RX_Clk), 
   .Ch7_RX_ER(Ch7_RX_ER),    .Ch7_RX_DV(Ch7_RX_DV), 
   .Ch8_RX_D(Ch8_RX_D),      .Ch8_RX_Clk(Ch8_RX_Clk),  
   .Ch8_RX_ER(Ch8_RX_ER),    .Ch8_RX_DV(Ch8_RX_DV), 
   .PatternSel(PatternSel),  .LatTrim(LatTrim),
   .Pattern1(Pattern1), .Pattern2(Pattern2), .Pattern3(Pattern3), .Pattern4(Pattern4), 
   .Pattern5(Pattern5), .Pattern6(Pattern6), .Pattern7(Pattern7), .Pattern8(Pattern8), 
   .qie1data(QIE1Data),    .qie2data(QIE2Data),    .qie3data(QIE3Data), 
   .qie4data(QIE4Data),    .qie5data(QIE5Data),    .qie6data(QIE6Data), 
   .qie7data(QIE7Data),    .qie8data(QIE8Data),    .qie9data(QIE9Data), 
   .qie10data(QIE10Data),  .qie11data(QIE11Data),  .qie12data(QIE12Data), 
   .qie13data(QIE13Data),  .qie14data(QIE14Data),  .qie15data(QIE15Data), 
   .qie16data(QIE16Data),  .qie17data(QIE17Data),  .qie18data(QIE18Data), 
   .qie19data(QIE19Data),  .qie20data(QIE20Data),  .qie21data(QIE21Data), 
   .qie22data(QIE22Data),  .qie23data(QIE23Data),  .qie24data(QIE24Data), 
   .InputSpyData1(InputSpyData1),  .InputSpyData2(InputSpyData2), 
   .InputSpyData3(InputSpyData3),  .InputSpyData4(InputSpyData4), 
   .InputSpyData5(InputSpyData5),  .InputSpyData6(InputSpyData6), 
   .InputSpyData7(InputSpyData7),  .InputSpyData8(InputSpyData8),
   .ErrAcc1(ErrAcc1), .ErrAcc2(ErrAcc2), .ErrAcc3(ErrAcc3), .ErrAcc4(ErrAcc4),
   .ErrAcc5(ErrAcc5), .ErrAcc6(ErrAcc6), .ErrAcc7(ErrAcc7), .ErrAcc8(ErrAcc8),
   .FifoFull(AFifoFull),  .FifoEmpty(AFifoEmpty),    .LW(LW),
   .Fifo1Status(AFifo1Status),    .Fifo2Status(AFifo2Status), 
   .Fifo3Status(AFifo3Status),    .Fifo4Status(AFifo4Status), 
   .Fifo5Status(AFifo5Status),    .Fifo6Status(AFifo6Status), 
   .Fifo7Status(AFifo7Status),    .Fifo8Status(AFifo8Status), 
   .WrEn(WrEn), .RdEn(RdEn),  .ProgInputDelay(ProgInputDelay) , 
   .resetTP1(resetTP1), .resetTP2(resetTP2), .resetTP3(resetTP3), .resetTP4(resetTP4),
   .resetTP5(resetTP5), .resetTP6(resetTP6), .resetTP7(resetTP7), .resetTP8(resetTP8),
   .Fib1BZero(Fib1BZero), .Fib2BZero(Fib2BZero), 
   .Fib3BZero(Fib3BZero), .Fib4BZero(Fib4BZero), 
   .Fib5BZero(Fib5BZero), .Fib6BZero(Fib6BZero),
   .Fib7BZero(Fib7BZero), .Fib8BZero(Fib8BZero)      );  // end InputStage inst.

// trigger_path port map. Format:  .InternalPortName(ext_port_expr)
TriggerPath   TriggerPath (
 .CLK(clk), 
 .QIE1Data(QieChMask[1]   ? 7'b0 : QIE1Data[6:0]),  .QIE2Data(QieChMask[2]   ? 7'b0 : QIE2Data[6:0]),
 .QIE3Data(QieChMask[3]   ? 7'b0 : QIE3Data[6:0]),  .QIE4Data(QieChMask[4]   ? 7'b0 : QIE4Data[6:0]),
 .QIE5Data(QieChMask[5]   ? 7'b0 : QIE5Data[6:0]),  .QIE6Data(QieChMask[6]   ? 7'b0 : QIE6Data[6:0]),
 .QIE7Data(QieChMask[7]   ? 7'b0 : QIE7Data[6:0]),  .QIE8Data(QieChMask[8]   ? 7'b0 : QIE8Data[6:0]),
 .QIE9Data(QieChMask[9]   ? 7'b0 : QIE9Data[6:0]),  .QIE10Data(QieChMask[10] ? 7'b0 : QIE10Data[6:0]),
 .QIE11Data(QieChMask[11] ? 7'b0 : QIE11Data[6:0]), .QIE12Data(QieChMask[12] ? 7'b0 : QIE12Data[6:0]), 
 .QIE13Data(QieChMask[13] ? 7'b0 : QIE13Data[6:0]), .QIE14Data(QieChMask[14] ? 7'b0 : QIE14Data[6:0]),
 .QIE15Data(QieChMask[15] ? 7'b0 : QIE15Data[6:0]), .QIE16Data(QieChMask[16] ? 7'b0 : QIE16Data[6:0]),
 .QIE17Data(QieChMask[17] ? 7'b0 : QIE17Data[6:0]), .QIE18Data(QieChMask[18] ? 7'b0 : QIE18Data[6:0]),
 .QIE19Data(QieChMask[19] ? 7'b0 : QIE19Data[6:0]), .QIE20Data(QieChMask[20] ? 7'b0 : QIE20Data[6:0]),
 .QIE21Data(QieChMask[21] ? 7'b0 : QIE21Data[6:0]), .QIE22Data(QieChMask[22] ? 7'b0 : QIE22Data[6:0]),
 .QIE23Data(QieChMask[23] ? 7'b0 : QIE23Data[6:0]), .QIE24Data(QieChMask[24] ? 7'b0 : QIE24Data[6:0]), 
    
 .InLut1DOut(InLut1DOut),     .InLut2DOut(InLut2DOut),    .InLut3DOut(InLut3DOut), 
 .InLut4DOut(InLut4DOut),     .InLut5DOut(InLut5DOut),    .InLut6DOut(InLut6DOut), 
 .InLut7DOut(InLut7DOut),     .InLut8DOut(InLut8DOut),    .InLut9DOut(InLut9DOut), 
 .InLut10DOut(InLut10DOut),   .InLut11DOut(InLut11DOut),  .InLut12DOut(InLut12DOut), 
 .InLut13DOut(InLut13DOut),   .InLut14DOut(InLut14DOut),  .InLut15DOut(InLut15DOut), 
 .InLut16DOut(InLut16DOut),   .InLut17DOut(InLut17DOut),  .InLut18DOut(InLut18DOut), 
 .InLut19DOut(InLut19DOut),   .InLut20DOut(InLut20DOut),  .InLut21DOut(InLut21DOut), 
 .InLut22DOut(InLut22DOut),   .InLut23DOut(InLut23DOut),  .InLut24DOut(InLut24DOut), 
 .InLUT_Addr(InLUT_Addr),     .InLUT_Data(InLUT_Data), 
 .InLUT_WE(InLUT_WE),         .InLUT_Mode(InLUT_Mode), 
                            
 .OutLUT_DOut1(OutLUT_DOut1),   .OutLUT_DOut2(OutLUT_DOut2),   .OutLUT_DOut3(OutLUT_DOut3), 
 .OutLUT_DOut4(OutLUT_DOut4),   .OutLUT_DOut5(OutLUT_DOut5),   .OutLUT_DOut6(OutLUT_DOut6), 
 .OutLUT_DOut7(OutLUT_DOut7),   .OutLUT_DOut8(OutLUT_DOut8),   .OutLUT_DOut9(OutLUT_DOut9), 
 .OutLUT_DOut10(OutLUT_DOut10), .OutLUT_DOut11(OutLUT_DOut11), .OutLUT_DOut12(OutLUT_DOut12), 
 .OutLUT_DOut13(OutLUT_DOut13), .OutLUT_DOut14(OutLUT_DOut14), .OutLUT_DOut15(OutLUT_DOut15), 
 .OutLUT_DOut16(OutLUT_DOut16), .OutLUT_DOut17(OutLUT_DOut17), .OutLUT_DOut18(OutLUT_DOut18), 
 .OutLUT_DOut19(OutLUT_DOut19), .OutLUT_DOut20(OutLUT_DOut20), .OutLUT_DOut21(OutLUT_DOut21), 
 .OutLUT_DOut22(OutLUT_DOut22), .OutLUT_DOut23(OutLUT_DOut23), .OutLUT_DOut24(OutLUT_DOut24), 
 .OutLUT_Addr(OutLUT_Addr),     .OutLUT_Data(OutLUT_Data), 
 .OutLUT_WE(OutLUT_WE),         .OutLUT_Mode(OutLUT_Mode), 
    
  .TP1_Data(TP1_DataINT),   .TP2_Data(TP2_DataINT),   .TP3_Data(TP3_DataINT),
  .TP4_Data(TP4_DataINT),   .TP5_Data(TP5_DataINT),   .TP6_Data(TP6_DataINT),
  .TP7_Data(TP7_DataINT),   .TP8_Data(TP8_DataINT),   .TP9_Data(TP9_DataINT),
  .TP10_Data(TP10_DataINT), .TP11_Data(TP11_DataINT), .TP12_Data(TP12_DataINT), 
  .TP13_Data(TP13_DataINT), .TP14_Data(TP14_DataINT), .TP15_Data(TP15_DataINT),
  .TP16_Data(TP16_DataINT), .TP17_Data(TP17_DataINT), .TP18_Data(TP18_DataINT),
  .TP19_Data(TP19_DataINT), .TP20_Data(TP20_DataINT), .TP21_Data(TP21_DataINT),
  .TP22_Data(TP22_DataINT), .TP23_Data(TP23_DataINT), .TP24_Data(TP24_DataINT),
  .bypass(bypass),
  .resetTP1(resetTP1), .resetTP2(resetTP2),
  .resetTP3(resetTP3), .resetTP4(resetTP4),
  .resetTP5(resetTP5), .resetTP6(resetTP6),
  .resetTP7(resetTP7), .resetTP8(resetTP8)              );

 reg PatternSel_d, PatternSel_2d, PatternSel_3d, PatternSel_4d, PatternSelD;
 reg DInError, LWbit; 
 always @ (posedge clk)    begin  
   // delay should match latency from PatternRAMs to TP*_DataINT
    PatternSel_d  <= PatternSel;    PatternSel_2d <= PatternSel_d;
    PatternSel_3d <= PatternSel_2d; PatternSel_4d <= PatternSel_3d;
    PatternSelD   <= PatternSel_4d;
    DInError <= (~QieChMask[3]  & resetTP1) | (~QieChMask[6]  & resetTP2) |
                (~QieChMask[9]  & resetTP3) | (~QieChMask[12] & resetTP4) |
                (~QieChMask[15] & resetTP5) | (~QieChMask[18] & resetTP6) |
                (~QieChMask[21] & resetTP7) | (~QieChMask[24] & resetTP8);
   // NB: DInError is always asserted after IDLEs (e.g. in the gap)
    LWbit    <= (~QieChMask[3]  & LW[1]) | (~QieChMask[6]  & LW[2]) |
                (~QieChMask[9]  & LW[3]) | (~QieChMask[12] & LW[4]) |
                (~QieChMask[15] & LW[5]) | (~QieChMask[18] & LW[6]) |
                (~QieChMask[21] & LW[7]) | (~QieChMask[24] & LW[8]) ;
 end

  // L2-DAQ path port map. Format:  .InternalPortName(ext_port_expr)
 DaqPath  DaqPath        (
  .QIE1Data(QIE1Data),   .QIE2Data(QIE2Data),   .QIE3Data(QIE3Data),   .QIE4Data(QIE4Data), 
  .QIE5Data(QIE5Data),   .QIE6Data(QIE6Data),   .QIE7Data(QIE7Data),   .QIE8Data(QIE8Data), 
  .QIE9Data(QIE9Data),   .QIE10Data(QIE10Data), .QIE11Data(QIE11Data), .QIE12Data(QIE12Data),
  .QIE13Data(QIE13Data), .QIE14Data(QIE14Data), .QIE15Data(QIE15Data), .QIE16Data(QIE16Data),
  .QIE17Data(QIE17Data), .QIE18Data(QIE18Data), .QIE19Data(QIE19Data), .QIE20Data(QIE20Data),
  .QIE21Data(QIE21Data), .QIE22Data(QIE22Data), .QIE23Data(QIE23Data), .QIE24Data(QIE24Data),
  .TP1_Data({1'b0, PatternSelD, TP1_DataINT}),   .TP2_Data({1'b0, PatternSelD, TP2_DataINT}),
  .TP3_Data({1'b0, PatternSelD, TP3_DataINT}),   .TP4_Data({1'b0, PatternSelD, TP4_DataINT}),
  .TP5_Data({1'b0, PatternSelD, TP5_DataINT}),   .TP6_Data({1'b0, PatternSelD, TP6_DataINT}),
  .TP7_Data({1'b0, PatternSelD, TP7_DataINT}),   .TP8_Data({1'b0, PatternSelD, TP8_DataINT}),
  .TP9_Data({1'b0, PatternSelD, TP9_DataINT}),   .TP10_Data({1'b0, PatternSelD, TP10_DataINT}),
  .TP11_Data({1'b0, PatternSelD, TP11_DataINT}), .TP12_Data({1'b0, PatternSelD, TP12_DataINT}), 
  .TP13_Data({1'b0, PatternSelD, TP13_DataINT}), .TP14_Data({1'b0, PatternSelD, TP14_DataINT}),
  .TP15_Data({1'b0, PatternSelD, TP15_DataINT}), .TP16_Data({1'b0, PatternSelD, TP16_DataINT}),
  .TP17_Data({1'b0, PatternSelD, TP17_DataINT}), .TP18_Data({1'b0, PatternSelD, TP18_DataINT}),
  .TP19_Data({1'b0, PatternSelD, TP19_DataINT}), .TP20_Data({1'b0, PatternSelD, TP20_DataINT}),
  .TP21_Data({1'b0, PatternSelD, TP21_DataINT}), .TP22_Data({1'b0, PatternSelD, TP22_DataINT}),
  .TP23_Data({1'b0, PatternSelD, TP23_DataINT}), .TP24_Data({1'b0, PatternSelD, TP24_DataINT}), 
  .Threshold1(Threshold1),   .Threshold2(Threshold2),   .Threshold3(Threshold3),
  .Threshold4(Threshold4),   .Threshold5(Threshold5),   .Threshold6(Threshold6), 
  .Threshold7(Threshold7),   .Threshold8(Threshold8),   .Threshold9(Threshold9), 
  .Threshold10(Threshold10), .Threshold11(Threshold11), .Threshold12(Threshold12), 
  .Threshold13(Threshold13), .Threshold14(Threshold14), .Threshold15(Threshold15),  
  .Threshold16(Threshold16), .Threshold17(Threshold17), .Threshold18(Threshold18), 
  .Threshold19(Threshold19), .Threshold20(Threshold20), .Threshold21(Threshold21), 
  .Threshold22(Threshold22), .Threshold23(Threshold23), .Threshold24(Threshold24), 
  .PipeLength(PipeLength[7:0]),   .NS(NS),  .NPS(NPS), .DInError(DInError),
  .CounterModeSel(CounterModeSel),     .PatternMode(PatternMode), 
  .EVN(EvN), .CTrEvN(CTrEvN), .BCN(BCN),  .OrN(OrN),  .CalibTrig(CalibTrig), 
  .L1A(L1A), .CompactNorm(CompactNorm), .CompactCalib(CompactCalib),            
  .EventBufferEmpty(EventBufferEmpty),     .HCN(HTRsubmodN[9:0]),
  .DataOut(DataOut),  .L2IDLE_b(L2IDLE_b),   .StatusBits(StatusBits),   
  .FifoFull(AFifoFull),  .FifoEmpty(AFifoEmpty), .LW(LWbit), .DLL_lock(DLL_lock),
  .Fifo1Status(AFifo1Status),    .Fifo2Status(AFifo2Status), 
  .Fifo3Status(AFifo3Status),    .Fifo4Status(AFifo4Status), 
  .Fifo5Status(AFifo5Status),    .Fifo6Status(AFifo6Status), 
  .Fifo7Status(AFifo7Status),    .Fifo8Status(AFifo8Status), 
  .StRq(StRq),       .UnlockCnt(UnlockCnt[1:0]),    .TTC_Ready(TTC_Ready), 
  .Fib1BZero(Fib1BZero), .Fib2BZero(Fib2BZero), .Fib3BZero(Fib3BZero), .Fib4BZero(Fib4BZero), 
  .Fib5BZero(Fib5BZero), .Fib6BZero(Fib6BZero), .Fib7BZero(Fib7BZero), .Fib8BZero(Fib8BZero),
  .BCNofFib1BZero(BCNofFib1BZero), .BCNofFib2BZero(BCNofFib2BZero), 
  .BCNofFib3BZero(BCNofFib3BZero), .BCNofFib4BZero(BCNofFib4BZero), 
  .BCNofFib5BZero(BCNofFib5BZero), .BCNofFib6BZero(BCNofFib6BZero),
  .BCNofFib7BZero(BCNofFib7BZero), .BCNofFib8BZero(BCNofFib8BZero), 
  .clk(clk),   .RST(Soft_rst),  .L2frameINT(L2frameINT)    );  // end DaqPath

   // Main Controller port map. Format:  .InternalPortName(ext_port_expr)
 MainCntr  MainCntr  (
  .clk(clk),                       .clk2x(clk2x),
  .ClockSelect(ClockSelect),       .AsyncFifoRst(AsyncFifoRst),
  .TTC_BCntRes(TTC_BCntRes),       .TTC_Brcst(TTC_Brcst),
  .TTC_BrcstStr(TTC_BrcstStr),     .TTC_CK40Des1(TTC_CK), 
  .TTC_Ready(TTC_Ready),           .TTC_Reset_b(TTC_Reset_b),
  .TTC_EvCntRes(TTC_EvCntRes),     .TTC_L1Accept(TTC_L1Accept),
  .TTC_DbErrStr(TTC_DbErrStr),     .TTC_SinErrStr(TTC_SinErrStr),
  .LocAddr(LocAddr),               .LocData(LocData),
  .LocCS_b(LocCS_b),               .LocWRITE_b(LocWRITE_b),
  .InputSpyData1(InputSpyData1),   .InputSpyData2(InputSpyData2), 
  .InputSpyData3(InputSpyData3),   .InputSpyData4(InputSpyData4),
  .InputSpyData5(InputSpyData5),   .InputSpyData6(InputSpyData6),
  .InputSpyData7(InputSpyData7),   .InputSpyData8(InputSpyData8),
  .L2Data(DataOut),  .L2StatusBits(StatusBits), .SD(SD), .HTRsubmodN(HTRsubmodN), 
  .event_buffer_empty(EventBufferEmpty),     .L2IDLE_b(L2IDLE_b),  
  .PatternMode(PatternMode),  .PatternSel(PatternSel), 
  .VME_TrigOut(),      .QieChMask(QieChMask),  .LatTrim(LatTrim),
  .PipeLength(PipeLength), .DLL_lock(DLL_lock),   .UnlockCnt(UnlockCnt),
  .CompactNorm(CompactNorm), .CompactCalib(CompactCalib), 
  .NS(NS),    .NPS(NPS),   .TI_EN(TI_EN), 
  .LB_STATE(LB_STATE), .LB_DOUT(LB_DOUT), .LB_Tri(LB_Tri),
  .BCN(BCN), .EvN(EvN), .CTrEvN(CTrEvN),  .OrN(OrN),
  .Hard_rst(Hard_rst), .Soft_rst(Soft_rst), .Start(Start),  .Stop(Stop), 
  .L1A(L1A), .TTC_BC0(TTC_BC0), .StRq(StRq), .LastTTCcommand(LastTTCcommand), 
  .run_status(run_status), .TTC_TestEnable(TTC_TestEnable),  .CalibTrig(CalibTrig), 
  .ErrAcc1(ErrAcc1), .ErrAcc2(ErrAcc2), .ErrAcc3(ErrAcc3), .ErrAcc4(ErrAcc4),
  .ErrAcc5(ErrAcc5), .ErrAcc6(ErrAcc6), .ErrAcc7(ErrAcc7), .ErrAcc8(ErrAcc8),
  .bypass(bypass),  .CounterModeSel(CounterModeSel),  .ProgInputDelay(ProgInputDelay),
  .AFifoFull(AFifoFull),  .AFifoEmpty(AFifoEmpty),    .LW(LW),
  .AFifo1Status(AFifo1Status),    .AFifo2Status(AFifo2Status), 
  .AFifo3Status(AFifo3Status),    .AFifo4Status(AFifo4Status), 
  .AFifo5Status(AFifo5Status),    .AFifo6Status(AFifo6Status), 
  .AFifo7Status(AFifo7Status),    .AFifo8Status(AFifo8Status), 
  .BCNofFib1BZero(BCNofFib1BZero), .BCNofFib2BZero(BCNofFib2BZero), 
  .BCNofFib3BZero(BCNofFib3BZero), .BCNofFib4BZero(BCNofFib4BZero), 
  .BCNofFib5BZero(BCNofFib5BZero), .BCNofFib6BZero(BCNofFib6BZero),
  .BCNofFib7BZero(BCNofFib7BZero), .BCNofFib8BZero(BCNofFib8BZero),    
  .InLUT_DOut1(InLut1DOut),     .InLUT_DOut2(InLut2DOut),     .InLUT_DOut3(InLut3DOut), 
  .InLUT_DOut4(InLut4DOut),     .InLUT_DOut5(InLut5DOut),     .InLUT_DOut6(InLut6DOut), 
  .InLUT_DOut7(InLut7DOut),     .InLUT_DOut8(InLut8DOut),     .InLUT_DOut9(InLut9DOut), 
  .InLUT_DOut10(InLut10DOut),   .InLUT_DOut11(InLut11DOut),   .InLUT_DOut12(InLut12DOut), 
  .InLUT_DOut13(InLut13DOut),   .InLUT_DOut14(InLut14DOut),   .InLUT_DOut15(InLut15DOut), 
  .InLUT_DOut16(InLut16DOut),   .InLUT_DOut17(InLut17DOut),   .InLUT_DOut18(InLut18DOut), 
  .InLUT_DOut19(InLut19DOut),   .InLUT_DOut20(InLut20DOut),   .InLUT_DOut21(InLut21DOut), 
  .InLUT_DOut22(InLut22DOut),   .InLUT_DOut23(InLut23DOut),   .InLUT_DOut24(InLut24DOut), 
  .InLUT_Addr(InLUT_Addr),       .InLUT_Data(InLUT_Data), 
  .InLUT_WE(InLUT_WE),           .InLUT_Mode(InLUT_Mode), 
  .OutLUT_DOut1(OutLUT_DOut1),   .OutLUT_DOut2(OutLUT_DOut2),   .OutLUT_DOut3(OutLUT_DOut3), 
  .OutLUT_DOut4(OutLUT_DOut4),   .OutLUT_DOut5(OutLUT_DOut5),   .OutLUT_DOut6(OutLUT_DOut6), 
  .OutLUT_DOut7(OutLUT_DOut7),   .OutLUT_DOut8(OutLUT_DOut8),   .OutLUT_DOut9(OutLUT_DOut9), 
  .OutLUT_DOut10(OutLUT_DOut10), .OutLUT_DOut11(OutLUT_DOut11), .OutLUT_DOut12(OutLUT_DOut12), 
  .OutLUT_DOut13(OutLUT_DOut13), .OutLUT_DOut14(OutLUT_DOut14), .OutLUT_DOut15(OutLUT_DOut15), 
  .OutLUT_DOut16(OutLUT_DOut16), .OutLUT_DOut17(OutLUT_DOut17), .OutLUT_DOut18(OutLUT_DOut18), 
  .OutLUT_DOut19(OutLUT_DOut19), .OutLUT_DOut20(OutLUT_DOut20), .OutLUT_DOut21(OutLUT_DOut21), 
  .OutLUT_DOut22(OutLUT_DOut22), .OutLUT_DOut23(OutLUT_DOut23), .OutLUT_DOut24(OutLUT_DOut24), 
  .OutLUT_Addr(OutLUT_Addr),     .OutLUT_Data(OutLUT_Data), 
  .OutLUT_WE(OutLUT_WE),         .OutLUT_Mode(OutLUT_Mode),
  .Pattern1(Pattern1), .Pattern2(Pattern2), .Pattern3(Pattern3), .Pattern4(Pattern4), 
  .Pattern5(Pattern5), .Pattern6(Pattern6), .Pattern7(Pattern7), .Pattern8(Pattern8),
  .Threshold1(Threshold1),   .Threshold2(Threshold2),   .Threshold3(Threshold3),
  .Threshold4(Threshold4),   .Threshold5(Threshold5),   .Threshold6(Threshold6), 
  .Threshold7(Threshold7),   .Threshold8(Threshold8),   .Threshold9(Threshold9), 
  .Threshold10(Threshold10), .Threshold11(Threshold11), .Threshold12(Threshold12), 
  .Threshold13(Threshold13), .Threshold14(Threshold14), .Threshold15(Threshold15),  
  .Threshold16(Threshold16), .Threshold17(Threshold17), .Threshold18(Threshold18), 
  .Threshold19(Threshold19), .Threshold20(Threshold20), .Threshold21(Threshold21), 
  .Threshold22(Threshold22), .Threshold23(Threshold23), .Threshold24(Threshold24) 
     );    // close MainCntr instantiation

/*   The STARTUP_VIRTEX2 primitive is used for Global Set/Reset (GSR), global 3-state control,
and the user configuration clock. The GSR input, when High,
sets or resets all flip-flops, all latches, and every block RAMB16 output register 
in the device, depending on the initialization state (INIT=1 or 0) of the component.
To synchronize startup to a user clock, connect the user clock signal to the CLK
input, as shown here. Furthermore, user clock must be selected in the BitGen program. */
// STARTUP_VIRTEX2 STARTUP_VIRTEX2 (.CLK (clk),  .GSR (), .GTS ());
     
// ******* SOURCE TERMINATION BUFFERS for outputs to ChannelLink and SLBs  ********
// OBUF_LVDCI_DV2_25 means: 
//   Output BUFfer Low-Voltage Digitally Controlled Inpedence DiVided by 2 at 2.5V
// These buffers insert a R=(On-board reference resistors DIV 2) =(60/2) = 30 Ohm

//////////  L2frame  resistors
   OBUF_LVDCI_DV2_25  L2buf0  (.I(L2frameINT[0]),  .O(L2frame[0]) );  
   OBUF_LVDCI_DV2_25  L2buf1  (.I(L2frameINT[1]),  .O(L2frame[1]) ); 
   OBUF_LVDCI_DV2_25  L2buf2  (.I(L2frameINT[2]),  .O(L2frame[2]) ); 
   OBUF_LVDCI_DV2_25  L2buf3  (.I(L2frameINT[3]),  .O(L2frame[3]) ); 
   OBUF_LVDCI_DV2_25  L2buf4  (.I(L2frameINT[4]),  .O(L2frame[4]) );  
   OBUF_LVDCI_DV2_25  L2buf5  (.I(L2frameINT[5]),  .O(L2frame[5]) ); 
   OBUF_LVDCI_DV2_25  L2buf6  (.I(L2frameINT[6]),  .O(L2frame[6]) ); 
   OBUF_LVDCI_DV2_25  L2buf7  (.I(L2frameINT[7]),  .O(L2frame[7]) ); 
   OBUF_LVDCI_DV2_25  L2buf8  (.I(L2frameINT[8]),  .O(L2frame[8]) ); 
   OBUF_LVDCI_DV2_25  L2buf9  (.I(L2frameINT[9]),  .O(L2frame[9]) ); 
   OBUF_LVDCI_DV2_25  L2buf10 (.I(L2frameINT[10]), .O(L2frame[10]) );  
   OBUF_LVDCI_DV2_25  L2buf11 (.I(L2frameINT[11]), .O(L2frame[11]) ); 
   OBUF_LVDCI_DV2_25  L2buf12 (.I(L2frameINT[12]), .O(L2frame[12]) ); 
   OBUF_LVDCI_DV2_25  L2buf13 (.I(L2frameINT[13]), .O(L2frame[13]) ); 
   OBUF_LVDCI_DV2_25  L2buf14 (.I(L2frameINT[14]), .O(L2frame[14]) );  
   OBUF_LVDCI_DV2_25  L2buf15 (.I(L2frameINT[15]), .O(L2frame[15]) ); 
   OBUF_LVDCI_DV2_25  L2buf16 (.I(L2frameINT[16]), .O(L2frame[16]) ); 
   OBUF_LVDCI_DV2_25  L2buf17 (.I(L2frameINT[17]), .O(L2frame[17]) ); 
   OBUF_LVDCI_DV2_25  L2buf18 (.I(L2frameINT[18]), .O(L2frame[18]) ); 
   OBUF_LVDCI_DV2_25  L2buf19 (.I(L2frameINT[19]), .O(L2frame[19]) ); 
   OBUF_LVDCI_DV2_25  L2buf20 (.I(L2frameINT[20]), .O(L2frame[20]) );  
   OBUF_LVDCI_DV2_25  L2buf21 (.I(L2frameINT[21]), .O(L2frame[21]) ); 
   OBUF_LVDCI_DV2_25  L2buf22 (.I(L2frameINT[22]), .O(L2frame[22]) ); 
   OBUF_LVDCI_DV2_25  L2buf23 (.I(L2frameINT[23]), .O(L2frame[23]) ); 

//////////////  TP resistors
//
// July 5, 2005 - Drew
// send 2 bits per channel for the HF Luminosity project.  for the
// time being (and maybe forever) just use the 2 MSBs from the TPG
// LUT.  these bits will be sent at 80MHz
//
// first, make a 40MHz switching signal that is in phase with the
// 80MHz clock clk2x
//
reg data_select;
always @ (posedge clk2x) data_select = ~data_select;
//
// now make the mux for sending the bits.  the 2 bits are the 2
// MSB from the TPG LUT, which means bits 6 and 7 (bit 8 is the
// muon, or feature, bit)
//
`define HIGHBIT 7 
`define LOWBIT 6 
reg[35:0] outbit;
reg[9:0] tcount;
always @ (posedge clk) tcount <= tcount + 1;
always @ (posedge clk2x) begin
  if (data_select) begin
    //
	 // mux=1, send bit 7
	 //
	 outbit <= #1 {1'b1,L1A,tcount,
	 					TP24_DataINT[`HIGHBIT],
	               TP23_DataINT[`HIGHBIT],
	               TP22_DataINT[`HIGHBIT],
	               TP21_DataINT[`HIGHBIT],
	               TP20_DataINT[`HIGHBIT],
	               TP19_DataINT[`HIGHBIT],
	               TP18_DataINT[`HIGHBIT],
	               TP17_DataINT[`HIGHBIT],
	               TP16_DataINT[`HIGHBIT],
	               TP15_DataINT[`HIGHBIT],
	               TP14_DataINT[`HIGHBIT],
	               TP13_DataINT[`HIGHBIT],
	               TP12_DataINT[`HIGHBIT],
	               TP11_DataINT[`HIGHBIT],
	               TP10_DataINT[`HIGHBIT],
	               TP9_DataINT[`HIGHBIT],
	               TP8_DataINT[`HIGHBIT],
	               TP7_DataINT[`HIGHBIT],
	               TP6_DataINT[`HIGHBIT],
	               TP5_DataINT[`HIGHBIT],
	               TP4_DataINT[`HIGHBIT],
	               TP3_DataINT[`HIGHBIT],
	               TP2_DataINT[`HIGHBIT],
	               TP1_DataINT[`HIGHBIT]};
  end
  else begin
    //
	 // mux=0, send bit 6
	 //
	 outbit <= #1 {1'b0,L1A,~tcount,
	 					TP24_DataINT[`LOWBIT],
	               TP23_DataINT[`LOWBIT],
	               TP22_DataINT[`LOWBIT],
	               TP21_DataINT[`LOWBIT],
	               TP20_DataINT[`LOWBIT],
	               TP19_DataINT[`LOWBIT],
	               TP18_DataINT[`LOWBIT],
	               TP17_DataINT[`LOWBIT],
	               TP16_DataINT[`LOWBIT],
	               TP15_DataINT[`LOWBIT],
	               TP14_DataINT[`LOWBIT],
	               TP13_DataINT[`LOWBIT],
	               TP12_DataINT[`LOWBIT],
	               TP11_DataINT[`LOWBIT],
	               TP10_DataINT[`LOWBIT],
	               TP9_DataINT[`LOWBIT],
	               TP8_DataINT[`LOWBIT],
	               TP7_DataINT[`LOWBIT],
	               TP6_DataINT[`LOWBIT],
	               TP5_DataINT[`LOWBIT],
	               TP4_DataINT[`LOWBIT],
	               TP3_DataINT[`LOWBIT],
	               TP2_DataINT[`LOWBIT],
	               TP1_DataINT[`LOWBIT]};
  end
end

//////////////  TP resistors
// SLB 1
// at this time we will use SLB1 for the output, but this can change

   OBUF_LVDCI_DV2_25  DOUTbuf10 (.I(outbit[0]), .O(TP1_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf11 (.I(outbit[1]), .O(TP1_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf12 (.I(outbit[2]), .O(TP1_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf13 (.I(outbit[3]), .O(TP1_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf14 (.I(outbit[4]), .O(TP1_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf15 (.I(outbit[5]), .O(TP1_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf16 (.I(outbit[6]), .O(TP1_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf17 (.I(outbit[7]), .O(TP1_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf18 (.I(outbit[8]), .O(TP1_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf20 (.I(outbit[9]), .O(TP2_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf21 (.I(outbit[10]), .O(TP2_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf22 (.I(outbit[11]), .O(TP2_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf23 (.I(outbit[12]), .O(TP2_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf24 (.I(outbit[13]), .O(TP2_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf25 (.I(outbit[14]), .O(TP2_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf26 (.I(outbit[15]), .O(TP2_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf27 (.I(outbit[16]), .O(TP2_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf28 (.I(outbit[17]), .O(TP2_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf30 (.I(outbit[18]), .O(TP3_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf31 (.I(outbit[19]), .O(TP3_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf32 (.I(outbit[20]), .O(TP3_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf33 (.I(outbit[21]), .O(TP3_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf34 (.I(outbit[22]), .O(TP3_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf35 (.I(outbit[23]), .O(TP3_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf36 (.I(outbit[24]), .O(TP3_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf37 (.I(outbit[25]), .O(TP3_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf38 (.I(outbit[26]), .O(TP3_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf40 (.I(outbit[27]), .O(TP4_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf41 (.I(outbit[28]), .O(TP4_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf42 (.I(outbit[29]), .O(TP4_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf43 (.I(outbit[30]), .O(TP4_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf44 (.I(outbit[31]), .O(TP4_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf45 (.I(outbit[32]), .O(TP4_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf46 (.I(outbit[33]), .O(TP4_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf47 (.I(outbit[34]), .O(TP4_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf48 (.I(outbit[35]), .O(TP4_Data[8]) ); 

/*
   OBUF_LVDCI_DV2_25  DOUTbuf10 (.I(TP1_DataINT[0]), .O(TP1_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf11 (.I(TP1_DataINT[1]), .O(TP1_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf12 (.I(TP1_DataINT[2]), .O(TP1_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf13 (.I(TP1_DataINT[3]), .O(TP1_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf14 (.I(TP1_DataINT[4]), .O(TP1_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf15 (.I(TP1_DataINT[5]), .O(TP1_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf16 (.I(TP1_DataINT[6]), .O(TP1_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf17 (.I(TP1_DataINT[7]), .O(TP1_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf18 (.I(TP1_DataINT[8]), .O(TP1_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf20 (.I(TP2_DataINT[0]), .O(TP2_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf21 (.I(TP2_DataINT[1]), .O(TP2_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf22 (.I(TP2_DataINT[2]), .O(TP2_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf23 (.I(TP2_DataINT[3]), .O(TP2_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf24 (.I(TP2_DataINT[4]), .O(TP2_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf25 (.I(TP2_DataINT[5]), .O(TP2_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf26 (.I(TP2_DataINT[6]), .O(TP2_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf27 (.I(TP2_DataINT[7]), .O(TP2_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf28 (.I(TP2_DataINT[8]), .O(TP2_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf30 (.I(TP3_DataINT[0]), .O(TP3_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf31 (.I(TP3_DataINT[1]), .O(TP3_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf32 (.I(TP3_DataINT[2]), .O(TP3_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf33 (.I(TP3_DataINT[3]), .O(TP3_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf34 (.I(TP3_DataINT[4]), .O(TP3_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf35 (.I(TP3_DataINT[5]), .O(TP3_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf36 (.I(TP3_DataINT[6]), .O(TP3_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf37 (.I(TP3_DataINT[7]), .O(TP3_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf38 (.I(TP3_DataINT[8]), .O(TP3_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf40 (.I(TP4_DataINT[0]), .O(TP4_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf41 (.I(TP4_DataINT[1]), .O(TP4_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf42 (.I(TP4_DataINT[2]), .O(TP4_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf43 (.I(TP4_DataINT[3]), .O(TP4_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf44 (.I(TP4_DataINT[4]), .O(TP4_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf45 (.I(TP4_DataINT[5]), .O(TP4_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf46 (.I(TP4_DataINT[6]), .O(TP4_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf47 (.I(TP4_DataINT[7]), .O(TP4_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf48 (.I(TP4_DataINT[8]), .O(TP4_Data[8]) ); 
*/
// SLB 2

   OBUF_LVDCI_DV2_25  DOUTbuf50 (.I(TP5_DataINT[0]), .O(TP5_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf51 (.I(TP5_DataINT[1]), .O(TP5_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf52 (.I(TP5_DataINT[2]), .O(TP5_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf53 (.I(TP5_DataINT[3]), .O(TP5_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf54 (.I(TP5_DataINT[4]), .O(TP5_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf55 (.I(TP5_DataINT[5]), .O(TP5_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf56 (.I(TP5_DataINT[6]), .O(TP5_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf57 (.I(TP5_DataINT[7]), .O(TP5_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf58 (.I(TP5_DataINT[8]), .O(TP5_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf60 (.I(TP6_DataINT[0]), .O(TP6_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf61 (.I(TP6_DataINT[1]), .O(TP6_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf62 (.I(TP6_DataINT[2]), .O(TP6_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf63 (.I(TP6_DataINT[3]), .O(TP6_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf64 (.I(TP6_DataINT[4]), .O(TP6_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf65 (.I(TP6_DataINT[5]), .O(TP6_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf66 (.I(TP6_DataINT[6]), .O(TP6_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf67 (.I(TP6_DataINT[7]), .O(TP6_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf68 (.I(TP6_DataINT[8]), .O(TP6_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf70 (.I(TP7_DataINT[0]), .O(TP7_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf71 (.I(TP7_DataINT[1]), .O(TP7_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf72 (.I(TP7_DataINT[2]), .O(TP7_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf73 (.I(TP7_DataINT[3]), .O(TP7_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf74 (.I(TP7_DataINT[4]), .O(TP7_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf75 (.I(TP7_DataINT[5]), .O(TP7_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf76 (.I(TP7_DataINT[6]), .O(TP7_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf77 (.I(TP7_DataINT[7]), .O(TP7_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf78 (.I(TP7_DataINT[8]), .O(TP7_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf80 (.I(TP8_DataINT[0]), .O(TP8_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf81 (.I(TP8_DataINT[1]), .O(TP8_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf82 (.I(TP8_DataINT[2]), .O(TP8_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf83 (.I(TP8_DataINT[3]), .O(TP8_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf84 (.I(TP8_DataINT[4]), .O(TP8_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf85 (.I(TP8_DataINT[5]), .O(TP8_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf86 (.I(TP8_DataINT[6]), .O(TP8_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf87 (.I(TP8_DataINT[7]), .O(TP8_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf88 (.I(TP8_DataINT[8]), .O(TP8_Data[8]) ); 

// SLB 3

   OBUF_LVDCI_DV2_25  DOUTbuf90 (.I(TP9_DataINT[0]), .O(TP9_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf91 (.I(TP9_DataINT[1]), .O(TP9_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf92 (.I(TP9_DataINT[2]), .O(TP9_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf93 (.I(TP9_DataINT[3]), .O(TP9_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf94 (.I(TP9_DataINT[4]), .O(TP9_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf95 (.I(TP9_DataINT[5]), .O(TP9_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf96 (.I(TP9_DataINT[6]), .O(TP9_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf97 (.I(TP9_DataINT[7]), .O(TP9_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf98 (.I(TP9_DataINT[8]), .O(TP9_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf100 (.I(TP10_DataINT[0]), .O(TP10_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf101 (.I(TP10_DataINT[1]), .O(TP10_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf102 (.I(TP10_DataINT[2]), .O(TP10_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf103 (.I(TP10_DataINT[3]), .O(TP10_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf104 (.I(TP10_DataINT[4]), .O(TP10_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf105 (.I(TP10_DataINT[5]), .O(TP10_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf106 (.I(TP10_DataINT[6]), .O(TP10_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf107 (.I(TP10_DataINT[7]), .O(TP10_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf108 (.I(TP10_DataINT[8]), .O(TP10_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf110 (.I(TP11_DataINT[0]), .O(TP11_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf111 (.I(TP11_DataINT[1]), .O(TP11_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf112 (.I(TP11_DataINT[2]), .O(TP11_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf113 (.I(TP11_DataINT[3]), .O(TP11_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf114 (.I(TP11_DataINT[4]), .O(TP11_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf115 (.I(TP11_DataINT[5]), .O(TP11_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf116 (.I(TP11_DataINT[6]), .O(TP11_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf117 (.I(TP11_DataINT[7]), .O(TP11_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf118 (.I(TP11_DataINT[8]), .O(TP11_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf120 (.I(TP12_DataINT[0]), .O(TP12_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf121 (.I(TP12_DataINT[1]), .O(TP12_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf122 (.I(TP12_DataINT[2]), .O(TP12_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf123 (.I(TP12_DataINT[3]), .O(TP12_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf124 (.I(TP12_DataINT[4]), .O(TP12_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf125 (.I(TP12_DataINT[5]), .O(TP12_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf126 (.I(TP12_DataINT[6]), .O(TP12_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf127 (.I(TP12_DataINT[7]), .O(TP12_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf128 (.I(TP12_DataINT[8]), .O(TP12_Data[8]) ); 

// SLB 4

   OBUF_LVDCI_DV2_25  DOUTbuf130 (.I(TP13_DataINT[0]), .O(TP13_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf131 (.I(TP13_DataINT[1]), .O(TP13_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf132 (.I(TP13_DataINT[2]), .O(TP13_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf133 (.I(TP13_DataINT[3]), .O(TP13_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf134 (.I(TP13_DataINT[4]), .O(TP13_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf135 (.I(TP13_DataINT[5]), .O(TP13_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf136 (.I(TP13_DataINT[6]), .O(TP13_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf137 (.I(TP13_DataINT[7]), .O(TP13_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf138 (.I(TP13_DataINT[8]), .O(TP13_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf140 (.I(TP14_DataINT[0]), .O(TP14_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf141 (.I(TP14_DataINT[1]), .O(TP14_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf142 (.I(TP14_DataINT[2]), .O(TP14_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf143 (.I(TP14_DataINT[3]), .O(TP14_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf144 (.I(TP14_DataINT[4]), .O(TP14_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf145 (.I(TP14_DataINT[5]), .O(TP14_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf146 (.I(TP14_DataINT[6]), .O(TP14_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf147 (.I(TP14_DataINT[7]), .O(TP14_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf148 (.I(TP14_DataINT[8]), .O(TP14_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf150 (.I(TP15_DataINT[0]), .O(TP15_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf151 (.I(TP15_DataINT[1]), .O(TP15_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf152 (.I(TP15_DataINT[2]), .O(TP15_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf153 (.I(TP15_DataINT[3]), .O(TP15_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf154 (.I(TP15_DataINT[4]), .O(TP15_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf155 (.I(TP15_DataINT[5]), .O(TP15_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf156 (.I(TP15_DataINT[6]), .O(TP15_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf157 (.I(TP15_DataINT[7]), .O(TP15_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf158 (.I(TP15_DataINT[8]), .O(TP15_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf160 (.I(TP16_DataINT[0]), .O(TP16_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf161 (.I(TP16_DataINT[1]), .O(TP16_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf162 (.I(TP16_DataINT[2]), .O(TP16_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf163 (.I(TP16_DataINT[3]), .O(TP16_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf164 (.I(TP16_DataINT[4]), .O(TP16_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf165 (.I(TP16_DataINT[5]), .O(TP16_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf166 (.I(TP16_DataINT[6]), .O(TP16_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf167 (.I(TP16_DataINT[7]), .O(TP16_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf168 (.I(TP16_DataINT[8]), .O(TP16_Data[8]) ); 

// SLB 5
		  
   OBUF_LVDCI_DV2_25  DOUTbuf170 (.I(TP17_DataINT[0]), .O(TP17_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf171 (.I(TP17_DataINT[1]), .O(TP17_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf172 (.I(TP17_DataINT[2]), .O(TP17_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf173 (.I(TP17_DataINT[3]), .O(TP17_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf174 (.I(TP17_DataINT[4]), .O(TP17_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf175 (.I(TP17_DataINT[5]), .O(TP17_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf176 (.I(TP17_DataINT[6]), .O(TP17_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf177 (.I(TP17_DataINT[7]), .O(TP17_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf178 (.I(TP17_DataINT[8]), .O(TP17_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf180 (.I(TP18_DataINT[0]), .O(TP18_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf181 (.I(TP18_DataINT[1]), .O(TP18_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf182 (.I(TP18_DataINT[2]), .O(TP18_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf183 (.I(TP18_DataINT[3]), .O(TP18_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf184 (.I(TP18_DataINT[4]), .O(TP18_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf185 (.I(TP18_DataINT[5]), .O(TP18_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf186 (.I(TP18_DataINT[6]), .O(TP18_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf187 (.I(TP18_DataINT[7]), .O(TP18_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf188 (.I(TP18_DataINT[8]), .O(TP18_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf190 (.I(TP19_DataINT[0]), .O(TP19_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf191 (.I(TP19_DataINT[1]), .O(TP19_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf192 (.I(TP19_DataINT[2]), .O(TP19_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf193 (.I(TP19_DataINT[3]), .O(TP19_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf194 (.I(TP19_DataINT[4]), .O(TP19_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf195 (.I(TP19_DataINT[5]), .O(TP19_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf196 (.I(TP19_DataINT[6]), .O(TP19_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf197 (.I(TP19_DataINT[7]), .O(TP19_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf198 (.I(TP19_DataINT[8]), .O(TP19_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf200 (.I(TP20_DataINT[0]), .O(TP20_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf201 (.I(TP20_DataINT[1]), .O(TP20_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf202 (.I(TP20_DataINT[2]), .O(TP20_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf203 (.I(TP20_DataINT[3]), .O(TP20_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf204 (.I(TP20_DataINT[4]), .O(TP20_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf205 (.I(TP20_DataINT[5]), .O(TP20_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf206 (.I(TP20_DataINT[6]), .O(TP20_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf207 (.I(TP20_DataINT[7]), .O(TP20_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf208 (.I(TP20_DataINT[8]), .O(TP20_Data[8]) ); 

// SLB 6

   OBUF_LVDCI_DV2_25  DOUTbuf210 (.I(TP21_DataINT[0]), .O(TP21_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf211 (.I(TP21_DataINT[1]), .O(TP21_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf212 (.I(TP21_DataINT[2]), .O(TP21_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf213 (.I(TP21_DataINT[3]), .O(TP21_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf214 (.I(TP21_DataINT[4]), .O(TP21_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf215 (.I(TP21_DataINT[5]), .O(TP21_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf216 (.I(TP21_DataINT[6]), .O(TP21_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf217 (.I(TP21_DataINT[7]), .O(TP21_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf218 (.I(TP21_DataINT[8]), .O(TP21_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf220 (.I(TP22_DataINT[0]), .O(TP22_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf221 (.I(TP22_DataINT[1]), .O(TP22_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf222 (.I(TP22_DataINT[2]), .O(TP22_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf223 (.I(TP22_DataINT[3]), .O(TP22_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf224 (.I(TP22_DataINT[4]), .O(TP22_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf225 (.I(TP22_DataINT[5]), .O(TP22_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf226 (.I(TP22_DataINT[6]), .O(TP22_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf227 (.I(TP22_DataINT[7]), .O(TP22_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf228 (.I(TP22_DataINT[8]), .O(TP22_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf230 (.I(TP23_DataINT[0]), .O(TP23_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf231 (.I(TP23_DataINT[1]), .O(TP23_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf232 (.I(TP23_DataINT[2]), .O(TP23_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf233 (.I(TP23_DataINT[3]), .O(TP23_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf234 (.I(TP23_DataINT[4]), .O(TP23_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf235 (.I(TP23_DataINT[5]), .O(TP23_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf236 (.I(TP23_DataINT[6]), .O(TP23_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf237 (.I(TP23_DataINT[7]), .O(TP23_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf238 (.I(TP23_DataINT[8]), .O(TP23_Data[8]) ); 

   OBUF_LVDCI_DV2_25  DOUTbuf240 (.I(TP24_DataINT[0]), .O(TP24_Data[0]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf241 (.I(TP24_DataINT[1]), .O(TP24_Data[1]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf242 (.I(TP24_DataINT[2]), .O(TP24_Data[2]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf243 (.I(TP24_DataINT[3]), .O(TP24_Data[3]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf244 (.I(TP24_DataINT[4]), .O(TP24_Data[4]) );  
   OBUF_LVDCI_DV2_25  DOUTbuf245 (.I(TP24_DataINT[5]), .O(TP24_Data[5]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf246 (.I(TP24_DataINT[6]), .O(TP24_Data[6]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf247 (.I(TP24_DataINT[7]), .O(TP24_Data[7]) ); 
   OBUF_LVDCI_DV2_25  DOUTbuf248 (.I(TP24_DataINT[8]), .O(TP24_Data[8]) ); 

   // AND gates thus the synthesizer does not trim unused inputs and than complain.
  assign ANDsig1 = BC0_INT & TTC_EvCntRes & Stop &
                   TTC_Reset_del & TTC_DbErrStr & TTC_SinErrStr &
                   LB_Tri & (&LB_DOUT) & (&LB_STATE);
endmodule